/*======================================================
  this code runs a forecast competition between
  a) linear model
  b) standard LSTAR model
  c) weighted LSTAR model
  
  after estimating nonlinear parameters we optimise and 
  delete lags according to AIC

  this models dgdp (quarterly obs) as a linear function of lagged GDP and
  lagged yield curve slope (monthly) and nonlinear transition as
  function of 
  +++++++++++++++++
  ysl up to 12 lags
  +++++++++++++++++

  as a result it saves the following information in 
  Gauss matrices
  - RSS, beta, weights, f, confidence intervals

  the forecast evaluation will be done in 
  fore_comp_eval.g

  Ralf Becker, 9/3/2008
======================================================*/

library pgraph,cml;
#include cml.ext;


CMLset;
graphset;
gausset;

/* GLOBALS */
__arlag = 1;
__slstarlag = 1;
scale_gam0 = 1;
__xall = 0;      
__xx = 0;
__xdet = 0;




            /* import data */
            /* UK GDP, quartely, Q1 1955 to Q2 2006 */
            /* 206 observations */
load gdp[] = c:\YOURPATH\ukgdp.txt;
            /* calculate log differences */
dgdp   = 100 * (ln(trimr(gdp,1,0))-ln(trimr(gdp,0,1)));     /* first obs 60q1 */


            /* UK interest data, monthly, Jan 1960 to Oct 2006 */
            /* Col1: 3mths Tbill rate, Col2: 10yr Govt securities */
            /* 562 observations */
load inter[562,2] = c:\YOURPATH\ukintrates.txt;
i3m = inter[.,1];
i10y = inter[.,2];
ysl = i10y - i3m;       /* slope of yield curve */



{ydep,yexp} = embed(dgdp,4,1,1);    /* ydep: Q2 56 to Q2 06 */
                                    /* yexp:    Q1 56 ... Q2 55
                                                ...
                                                Q1 06 ... Q2 05 */

{rub,xexp} = embed(ysl,12,1,1);    /* rub: not needed */
                                    /* xexp:    Dec 60 ... Jan 60 
                                                ...
                                                Sep 06 ... Oct 05 */

/* Setup the observations such that the first observation has the following data:

dep var         exp var
dgdp(Q1 61)     dgdp(Q4 60) ... dgdp(Q1 60) ysl(Dec 60) ... ysl(Jan 06) 
...
dgdp(Q2 06)     dgdp(Q1 06) ... dgdp(Q2 05) ysl(Mar 06) ... ysl(Apr 05) 
*/

ydep = trimr(ydep,19,0);
yexp = trimr(yexp,19,0);
xexp = trimr(xexp,0,6);

sel = seqa(1,3,rows(ydep)); /* pick end of quarter */
xexp = xexp[sel,.];

d73 = zeros(rows(ydep),1); 
d73[49] = 1;                    /* blip dummy for Q1 1973, placed relative to 1st obs Q1 61 */
data = ydep;
xall = yexp~xexp;               /* max 4 lags for dgdp and 12 for ysl */
xx = xexp;                      /* matrix from which trans var is calculated */
xdet = ones(rows(data),1)~d73;           /* deterministic variables, incl. const. */

xall_1saf = xall[2:rows(xall),.];               /* xall for forecasting */
xall_1saf = ones(rows(xall_1saf),1)~xall_1saf;
xall      = xall[1:rows(xall)-1,.];             /* xall for estimation */
xx_1saf   = xx[2:rows(xx),.];                   /* xx for forecasting */
xx        = xx[1:rows(xx)-1,.];                 /* xx for estimation */
xdet_1saf = xdet[2:rows(xx),.];                 /* xdet for forecasting */
xdet      = xdet[1:rows(xx)-1,.];               /* xdet for estimation */
   
rndseed 123457;	  

tt = rows(xall);        /* total number of estimation observations */
t0 = 156;               /* initial estimation sample */
t_oos = 1;              /* forecasting length 
                        should be = 1 if exogenous RHS variables */
nfore = (tt-t0)/t_oos; 

maxlag = cols(xx);  /* max lag for transition variable */
p = 7;             /* lag for linear part; this is combined dgdp (4 lags) and ysl (3 monthly lags) */

                              /* grid search for starting values
                                for c, gam and starting values for
                                a and b in the beta pdf approximating 
                                the TV weights */
gam_grid = seqa(0.1,0.1,9)|seqa(1,10,100); 
lag_grid = seqa(1,1,maxlag);  /* note scaling of gam parameter in obj. functions */

                     /* matrices to save results */
save_pval = zeros(nfore,3);          /* saves p-values of WSTAR tests */
save_pval_hc = zeros(nfore,3);       /* saves p-values of robust WSTAR tests */
save_pval_wb = zeros(nfore,3);       /* saves p-values of wild bootstrap WSTAR tests */
save_pval_tera = zeros(nfore,1);     /* saves p-value of Terasvirta test for LSTAR */
save_best_lag = zeros(nfore,2);      /* save chosen lag from test (col 1) and from grid search (col 2) */
@save_sbc_lag = zeros(nfore,1);       /* saves best linear lag chosen by SBC criterion */@
save_percentile = zeros(nfore,1);    /* save percentile position of last observation in estimation sample */
save_yeval = zeros(nfore,t_oos);     /* save evaluation observations */
save_ylast = zeros(nfore,10);        /* save last 10 observations */
save_ypos = zeros(nfore,1);          /* save percentile position of last observation */

/*==============================================================================
    Model Legend:

    arI: linear AR(p)
    arII: linear AR(aic)
    lstarI: LSTAR model with AR(sbc) linear part, TV according to lin tests
    lstarII: LSTAR model with AR(sbc) linear part, TV according to grid search
    wstarI: weighted LSTAR model with linear AR(p) part
    wstarII: weighted LSTAR model with linear AR(sbc) part
==============================================================================*/

arI_rss = zeros(nfore,1);
arI_f = zeros(nfore,t_oos);
arI_ci5 = zeros(nfore,t_oos); arI_ci25 = arI_ci5; arI_ci75 = arI_ci5; arI_ci95 = arI_ci5;

arII_rss = zeros(nfore,1);
arII_f = zeros(nfore,t_oos);
arII_ci5 = zeros(nfore,t_oos); arII_ci25 = arII_ci5; arII_ci75 = arII_ci5; arII_ci95 = arII_ci5;

lstarI_rss =  zeros(nfore,1);
lstarI_f = zeros(nfore,t_oos);
lstarI_ci5 = zeros(nfore,t_oos); lstarI_ci25 = lstarI_ci5; lstarI_ci75 = lstarI_ci5; lstarI_ci95 = lstarI_ci5;
lstarII_rss =  zeros(nfore,1);
lstarII_f = zeros(nfore,t_oos);
lstarII_ci5 = zeros(nfore,t_oos); lstarII_ci25 = lstarII_ci5; lstarII_ci75 = lstarII_ci5; lstarII_ci95 = lstarII_ci5;

wstarI_rss =  zeros(nfore,1);
wstarI_f = zeros(nfore,t_oos);
wstarI_ci5 = zeros(nfore,t_oos); wstarI_ci25 = wstarI_ci5; wstarI_ci75 = wstarI_ci5; wstarI_ci95 = wstarI_ci5;
wstarI_beta = zeros(nfore,4);

wstarII_rss =  zeros(nfore,1);
wstarII_f = zeros(nfore,t_oos);
wstarII_ci5 = zeros(nfore,t_oos); wstarII_ci25 = wstarII_ci5; wstarII_ci75 = wstarII_ci5; wstarII_ci95 = wstarII_ci5;
wstarII_weights = zeros(nfore,cols(xx));
wstarII_beta = zeros(nfore,4);

ns = 1;

latest_wstarI_parest = zeros(4,1);

do while ns <= nfore;      /*  start recursive loop */
/*do while ns <= nfore;        start recursive loop */

  print "forecast period = " ns;

  y00 = data[1:t0+(ns-1)*t_oos];                    /* estimation sample */
  xall00 = xall[1:t0+(ns-1)*t_oos,.];               /* x for linear part, estimation sample */
  xx00 = xx[1:t0+(ns-1)*t_oos,.];                   /* xx is needed for the calculation of the transition variable */
  xall00_1saf = xall_1saf[t0+(ns-1)*t_oos,.];       /* extract the relevant matrices for 1 step ahead forecasts */
  xx00_1saf = xx_1saf[t0+(ns-1)*t_oos,.];
  xdet00 = xdet[1:t0+(ns-1)*t_oos,.];
  xdet00_1saf = xdet[t0+(ns-1)*t_oos,.];

  y_eval = data[t0+(ns-1)*t_oos+1:t0+ns*t_oos];     /* realisations for forecast evaluation */

  sdy = stdc(y00);
    
  save_yeval[ns,.] = y_eval';                       /* save evaluation observations */
  save_ylast[ns,.] = y00[rows(y00)-9:rows(y00)]';   /* save last 10 observations */
  save_ypos[ns] = meanc(y00[rows(y00)].>y00);       /* save percentile position of last observation */


  /*==========================
    first produce tests 
    for linearity 
  ===========================*/

  scale_gam = scale_gam0/stdc(y00);                       /* used throughout loop */

  {save_sbc_lag,save_aic_lag} = sel_lag2(y00,xdet00,xall00[.,1:7],0);    /* this gives the optimal lag lengths according to aic and sbc */
                                                                /* ysl is restricted to a max of 3 lags */
                                                                /* output is vector indexing the elements in xall that should survive */
  save_sel_lag = save_aic_lag;
/*
print "Optimal lags according to SBC:";
print save_sbc_lag;
*/

  selp = 1|2|3|5|6;                     /* selects dgdp(-1 to -3) and ysl(-1 to -2) */

/*  selp1 = 1|2|3|5;                       selects dgdp(-1 to -3) and ysl(-1) lin part for STAR */
/*  selp2 = 2|5|6;                         selects dgdp(-2) and ysl(-1 to -2) nonlin part for STAR */

  selp1 = selp;
  selp2 = selp;

  xxsel = seqa(5,1,12);                 /* selects all ysl from xall00 for Tera nonline test */

  x0 = xall00[.,selp];                    /* restrict linear model to selp */  
  x0 = xdet00~x0;
  x0_1saf = xdet00_1saf~xall00_1saf[1,selp];         /* already includes const */

  x0_sel = xall00[.,save_sel_lag];      /* this x with lags selected according to aic */
  x0_sel = xdet00~x0_sel;
  x0_sel_1saf = xdet00_1saf~xall00_1saf[1,save_sel_lag];  /* this x with lags selected according to SBC */

  {temp,optbetaq} = wSTAR(y00,xall00[.,selp],xdet00,xx00); 
  save_pval[ns,.] = temp[1:3,.]';
  save_pval_hc[ns,.] = temp[4:6,.]';
  save_pval_wb[ns,.] = temp[7:9,.]';
  {save_pval_tera[ns],save_best_lag[ns,1]} = tera(y00,xall00,xdet00,xxsel);
                                        /* the index saved in save_best_lag[ns,1] refers to
                                           the variables selected through xxsel */


  /*===========================
    Linear model forecasting 
  =============================*/
  beta0 = (y00/x0);                                      /* use AR(maxlag) */
  arI_rss[ns] = (y00-x0*beta0)'(y00-x0*beta0);
  {arI_f[ns,.],temp_ci} = ARfore_xx_mc(y00,x0,x0_1saf,beta0);
  arI_ci5[ns,.] = temp_ci[1,.]; arI_ci25[ns,.] = temp_ci[2,.]; arI_ci75[ns,.] = temp_ci[3,.]; arI_ci95[ns,.] = temp_ci[4,.];

  beta0_sel = (y00/x0_sel);                              /* use AR(aic) */
  arII_rss[ns,.] = (y00-x0_sel*beta0_sel)'(y00-x0_sel*beta0_sel);
  {arII_f[ns,.],temp_ci} = ARfore_xx_mc(y00,x0_sel,x0_sel_1saf,beta0_sel);
  arII_ci5[ns,.] = temp_ci[1,.]; arII_ci25[ns,.] = temp_ci[2,.]; arII_ci75[ns,.] = temp_ci[3,.]; arII_ci95[ns,.] = temp_ci[4,.];


  /*================================================================
    Standard LSTAR estimation 
    there will be two LSTAR models:
    I: TV lag chosen according to grid search, save_best_lag[ns,2]
    II: TV lag chosen according to lin tests, save_best_lag[ns,1]
    both models use AR(sbc) as their linear model
    ---------------------------
    (a) preliminary grid search
  ==================================================================*/

  c_grid = quantile(xx00[.,1],(0.10|0.90));   /* choose grid as fct of tv */
  c_grid = seqa(c_grid[1],(c_grid[2]-c_grid[1])/49,50);

                      /* AR(4) for linear part */
  __output = 0; _olsres = 0;

  sel = (1|1|1);        /* which gam and c and lag to select */
  sel_rest = (1|1);     /* which gam and c to select when the lag is determined from the LSTAR test */ 
  temp = 100*y00'y00;     /* initialise the RSS criterion */
  temp_rest = temp;

                      /* grid search over unidentified parameters */
  for i(1,rows(gam_grid),1);
  for j(1,rows(c_grid),1);
  for k(1,rows(lag_grid),1);

    tv = xx00[.,k];
    f = 1/(1+exp(-(gam_grid[i]*scale_gam)*(tv-c_grid[j])));  
    xtest = xdet00 ~ xall00[.,selp1] ~ f.*xdet00[.,1] ~ f.*xall00[.,selp2];  /* use AR with "selp" selection */
    /* xtest = xdet00 ~ xall00[.,selp] ~ f.*xdet00[.,1] ~ f.*xall00[.,selp];  use AR with "selp" selection */

                                                     /* ensure that dummy is not f.*dum */               
                                    /* record standard error of estimate as temp_new */
    { d,d,d,d,d,d,temp_new,d,d,d,d } = ols(0,y00,xtest);  
    test = (temp_new<temp);   /* test for improvement */
    temp = test*temp_new + (1-test)*temp;
    sel[1] = test*i + (1-test)*sel[1];
    sel[2] = test*j + (1-test)*sel[2];
    sel[3] = test*k + (1-test)*sel[3];

    @if (test == 1); print gam_grid[i]~c_grid[j]~lag_grid[k]; endif;@

    if k == save_best_lag[ns,1];      /* best parameters when lag is determined from test */
      test = (temp_new<temp_rest);   /* test for improvement */
      temp_rest = test*temp_new + (1-test)*temp_rest;
      sel_rest[1] = test*i + (1-test)*sel_rest[1];
      sel_rest[2] = test*j + (1-test)*sel_rest[2];
    endif;

  endfor;
  endfor;
  endfor;

  gam0 = gam_grid[sel[1]];                  /* save grid search parameters */
  c0   = c_grid[sel[2]]; 
  save_best_lag[ns,2] = lag_grid[sel[3]];   
  lag0 = save_best_lag[ns,2];

  print "Grid search parameters";
  print "Gam = " gam0;
  print "c   = " c0;
  print "lag = " lag0;

  gam0_rest = gam_grid[sel_rest[1]];
  c0_rest   = c_grid[sel_rest[2]]; 


  /*===========================
    Standard LSTAR estimation 
    (b) full optimisation
    (c) forecasting
  =============================*/
                        /* first optimise concentrated likelihood and then linear parameters */
  __title = "Optimise standard LSTAR";
                                /* first version according to grid search lag */                             
  par0_rest = (10|c0_rest);     /* reduce gamma for optimisation start */
   

/* CHECK LAG SELECTION HERE */

  __slstarlag = save_best_lag[ns,1];        /* this sets the delay lag for TV according to Terasvirta rule */
  __xall = xall00;                          /* hand in matrix of exp. var for linear part */       
  __xx = xx00;                              /* hand in the transition variable matrix */
  __xdet = xdet00;

  __selp1 = selp1;
  __selp2 = selp2;
/*
  __selp1 = selp;
  __selp2 = selp;
*/

  lb = quantile(xx00[.,save_best_lag[ns,1]],0.05); 
  ub = quantile(xx00[.,save_best_lag[ns,1]],0.95);
  _cml_Bounds = (0~1e256)|(lb~ub);
  _cml_MaxIters  = 1000;
  _cml_Algorithm = 3;
  _cml_LineSearch = 3;
  _cml_DirTol = 1e-5;
  __output = 0;

  { beta_lstar,rub,g,cov,retcode } = CML(y00,0,&lstar_crit_conc_xx,par0_rest);
  print "retcode LSTARII =" retcode ", Iterations =" _cml_IterData[1];
  tv = xx00[.,save_best_lag[ns,1]];            /* select transition variable according to Terasvirta rule */ 
  tvf = xx00_1saf[.,save_best_lag[ns,1]];      /* select transition variable according to Terasvirta rule */ 
  f = 1/(1+exp(-(beta_lstar[1]*scale_gam)*(tv-beta_lstar[2])));  
  xtest = xdet00 ~ xall00[.,selp1] ~ xdet00[.,1].*f ~ xall00[.,selp2].*f; 
  /*xtest = xdet00 ~ xall00[.,selp] ~ xdet00[.,1].*f ~ xall00[.,selp].*f;*/
                        /* now find linear parameters */
  __output = 0; _olsres = 1;

  {sbc1,aic1} = sel_lag2(y00,xdet00,xtest[.,3:cols(xtest)],3);  /* used to get optimal lags */
  __selp1 = selif(aic1, aic1 .< 6);      /* asuming that selp = 1|2|3|5|6 */
  __selp2 = selif(aic1, aic1 .> 6)-6; 
  selxtest = 1|2|(aic1+2);
  { d,d,beta_lstar_lin,d,d,d,d,d,d,res,d } = ols(0,y00,xtest[.,selxtest]); 

  lstarII_rss[ns] = res'res;

  {lstarII_f[ns,.],temp_ci} = LSTARfore_xx(y00,xdet00,xall00,tv,xdet00_1saf,xall00_1saf,tvf,beta_lstar_lin,beta_lstar);     /* LSTAR forecasts with LSTAR rule */
  lstarII_ci5[ns,.] = temp_ci[1,.]; lstarII_ci25[ns,.] = temp_ci[2,.]; lstarII_ci75[ns,.] = temp_ci[3,.]; lstarII_ci95[ns,.] = temp_ci[4,.];

  __selp1 = selp;
  __selp2 = selp;

                               /* here different lag choice requires new estimation */
                               /* version according to grid search lag */
  par0 = (10|c0);
  lb = quantile(xx00[.,save_best_lag[ns,2]],0.05); 
  ub = quantile(xx00[.,save_best_lag[ns,2]],0.95);
  _cml_Bounds = (0~1e256)|(lb~ub);
  __slstarlag = save_best_lag[ns,2];        /* this sets the best delay lag for TV according to grid search */
  __output = 0;

  { beta_lstar,rub,g,cov,retcode } = CML(y00,0,&lstar_crit_conc_xx,par0);
  print "retcode LSTARI =" retcode ", Iterations =" _cml_IterData[1];
  tv = xx00[.,save_best_lag[ns,2]];            /* select transition variable according to grid search */ 
  tvf = xx00_1saf[.,save_best_lag[ns,2]];      /* select transition variable according to Terasvirta rule */ 
  f = 1/(1+exp(-(beta_lstar[1]*scale_gam)*(tv-beta_lstar[2])));  
  xtest = xdet00 ~ xall00[.,selp1] ~ xdet00[.,1].*f ~ xall00[.,selp2].*f; 
/*  xtest = xdet00 ~ xall00[.,selp] ~ xdet00[.,1].*f ~ xall00[.,selp].*f; */
                        /* now find linear parameters */
  __output = 0; _olsres = 1;

  {sbc2,aic2} = sel_lag2(y00,xdet00,xtest[.,3:cols(xtest)],3);  /* used to get optimal lags */
  __selp1 = selif(aic2, aic2 .< 6);      /* asuming that selp = 1|2|3|5|6 */
  __selp2 = selif(aic2, aic2 .> 6)-6; 
  selxtest = 1|2|(aic2+2);
  { d,d,beta_lstar_lin,d,d,d,d,d,d,res,d } = ols(0,y00,xtest[.,selxtest]); 

  lstarI_rss[ns] = res'res;

  {lstarI_f[ns,.],temp_ci} = LSTARfore_xx(y00,xdet00,xall00,tv,xdet00_1saf,xall00_1saf,tvf,beta_lstar_lin,beta_lstar);     /* LSTAR forecasts with grid lag */
  lstarI_ci5[ns,.] = temp_ci[1,.]; lstarI_ci25[ns,.] = temp_ci[2,.]; lstarI_ci75[ns,.] = temp_ci[3,.]; lstarI_ci95[ns,.] = temp_ci[4,.];

  __selp1 = selp;
  __selp2 = selp;


  /*===============================================
    Weighted LSTAR estimation 
    there will be two WSTAR models
    II: linear lag length AR(sbc)
    ----------------------------
    (a) grid search for various beta parameters
        and use for forecasts, WSTARI
    (b) optimise over parameters
        and use for forecasts, WSTARII = best of a and b
  =================================================*/

                      /* (a) grid search over unidentified parameters */

                        /* beta parameters to choose from */
  betachoice = 1~1|8~45|16~45|16~25|15~15|65~65|25~16|45~16|45~8|11~99|16~98|27~91|44~99|62~99|85~99|99~85|99~62|99~44|91~27|98~16|99~11;
  rss_choice = y00'y00;
  beta_lin_choice = zeros(cols(beta_lstar_lin),1);
  xtest_choice = 0;

  for k(1,rows(betachoice),1);

    beta_k = betachoice[k,.];
    split = 1/(cols(xx00)+1);
    splitgrid = seqa(split,split,cols(xx00));
    weights = dbeta(splitgrid,beta_k[1],beta_k[2]);
    weights = weights/sumc(weights);
           
    tv = xx00*weights;                /* calculate weighted transition variable */

    for i(1,rows(gam_grid),1);
    for j(1,rows(c_grid),1);
  
      f = 1/(1+exp(-(gam_grid[i]*scale_gam)*(tv-c_grid[j])));  
      xtest = xdet00 ~ xall00[.,selp1] ~ f.*xdet00[.,1] ~ f.*xall00[.,selp2]; /* use AR with "selp" selection */
                                    /* ensure that dummy is not f.*dum */               
                                    /* record standard error of estimate as temp_new */
      { d,d,beta_lin_choice_temp,d,d,d,temp_new,d,d,res_choice,d } = ols(0,y00,xtest);  
      test = ((res_choice'res_choice)<rss_choice);   /* test for improvement */
      rss_choice = test*(res_choice'res_choice) + (1-test)*rss_choice;
      beta_lin_choice = test*beta_lin_choice_temp + (1-test)*beta_lin_choice;
      xtest_choice = test*xtest + (1-test)*xtest_choice;
      sel[1] = test*i + (1-test)*sel[1];
      sel[2] = test*j + (1-test)*sel[2];
      sel[3] = test*k + (1-test)*sel[3];

    endfor;
    endfor;
    endfor;

    {sbc3,aic3} = sel_lag2(y00,xdet00,xtest_choice[.,3:cols(xtest_choice)],4);  /* used to get optimal lags */
    __selp1 = selif(aic3, aic3 .< 6);      /* asuming that selp = 1|2|3|5|6, ie 5 elements, 6th is f.*const */
    __selp2 = selif(aic3, aic3 .> 6)-6; 
    selxtest = 1|2|(__selp1+2)|8|(__selp2+8);
    { d,d,beta_wstar_lin,d,d,d,d,d,d,res,d } = ols(0,y00,xtest_choice[.,selxtest]); 

    gam0 = gam_grid[sel[1]];                  /* save grid search parameters */
    c0   = c_grid[sel[2]]; 
    beta0 = betachoice[sel[3],.];
    wstarI_beta[ns,.] = (gam0~c0~beta0);
    wstarI_rss[ns] = rss_choice;

    {wstarI_f[ns,.],temp_ci} = WSTARfore_xx(y00,xdet00,xall00,xx00,xdet00_1saf,xall00_1saf,xx00_1saf,beta_wstar_lin,wstarI_beta[ns,.]);     /* WSTAR forecasts */
    wstarI_ci5[ns,.] = temp_ci[1,.]; wstarI_ci25[ns,.] = temp_ci[2,.]; wstarI_ci75[ns,.] = temp_ci[3,.]; wstarI_ci95[ns,.] = temp_ci[4,.];

    __selp1 = selp;
    __selp2 = selp;

                        /* WEIGHTED LSTAR */

    weights = zeros(maxlag,1);
    weights[save_best_lag[ns,2]] = 1;       @ use best weight from grid search @
    par0 = (1|1);                           @ start with uniform, a=1, b=1 @
    __title = "Search beta";
    _cml_Bounds = (0.01~40 | 0.01~40);        @ restrict a and b @
    __output = 0;
    { beta_wstar,d,d,d,retcode } = CML(weights,0,&optbetaweight,par0);

    a = beta_wstar[1];             @ these are the parameters which best represent full weight on lag0 @
    b = beta_wstar[2];

    __title = "Optimise weighted LSTAR";

    if ns == 1;
      latest_wstarII_parest = wstarI_beta[ns,.]';
    endif;

    par0select = zeros(4,7);            /* different starting values */
    par0select[.,1] = 1|c0|a|b;
    par0select[.,2] = 1|c0|0.5*a|0.5*b;
    par0select[.,3] = 1|c0|5|5;
    par0select[.,4] = 1|c0|1|1;
    par0select[.,5] = wstarI_beta[ns,.]';
    par0select[.,6] = latest_wstarII_parest;
    par0select[.,7] = 1|latest_wstarII_parest[2:4];
    
    for ii(1,7,1);

      par0 = par0select[.,ii];  /* select starting value */

            /* restrict gam, a and b to be > 0 */
      lb = quantile(y00,0.05); ub = quantile(y00,0.95);

      lb = quantile(xx00[.,save_best_lag[ns,2]],0.05); 
      ub = quantile(xx00[.,save_best_lag[ns,2]],0.95);

            /* ensure c0 is in permissable region */
      test = (par0[2]>ub).or(par0[2]<lb);
      if test == 1;
        par0[2] = (ub-lb)/2;
      endif;

      _cml_Bounds =  (0~1e256)|(lb~ub)|(0.01~100)|(0.01~100);
      __arlag = p;                           /* here: AR(p) for linear model */
      _cml_MaxIters  = 1000;
      __output = 0;

      print "par0       =" par0';
      { beta_wstar,d,d,d,retcode } = CML(y00,0,&wlstar_crit_conc_xx,par0);
      print "beta_wstar =" beta_wstar';
      print "retcode WSTARII =" retcode ", Iterations =" _cml_IterData[1];

            /* transform a and b into weights on [1,8] */
      split = 1/(cols(xx00)+1);
      splitgrid = seqa(split,split,cols(xx00));
      weights = dbeta(splitgrid,beta_wstar[3],beta_wstar[4]);
      weights = weights/sumc(weights);
           
      tv = xx00*weights;                /* calculate weighted transition variable */
                                    /* xx00 has maxlag lags */
      f = 1/(1+exp(-(beta_wstar[1]*scale_gam)*(tv-beta_wstar[2])));  
      xtest = __xdet ~ xall00[.,selp1] ~ f.*__xdet[.,1] ~ f.*xall00[.,selp2];             /* this uses AR(p) for linear model */

                        /* now find linear parameters */
      __output = 0; _olsres = 1;
      { d,d,beta_wstar_lin,d,d,d,d,d,d,res,d } = ols(0,y00,xtest); 

      if ii == 1;
        beta_wstar_temp = beta_wstar;
        wstarII_rss[ns] = res'res;
        wstarII_weights[ns,.] = weights';
      else;
        if (res'res) < wstarII_rss[ns];  /* improved fit */
          print "IMPROVEMENT! ";
          beta_wstar_temp = beta_wstar;
          wstarII_rss[ns] = res'res;
          wstarII_weights[ns,.] = weights';      
        endif;
      endif;

    endfor;     /* end ii loop */

    latest_wstarII_parest = beta_wstar_temp;



            /* plot weights */

begwind;
window(2,1,0);
setwind(1);
  ytics(0.0,0.5,0.1,0); 
  betaplot(beta_wstar_temp[3:4],cols(xx00),"Monthly weights for transition variable");

/*
nextwind;
  betaplot(optbetam,cols(tvar),"Monthly weights");
*/

nextwind;
  graphset;
  title("transition variable");
  xtics(1960,2000,4,0);
  xy(seqa(1961.00,0.25,rows(tv)),(tv~(ones(rows(tv),1)*beta_wstar[2])));

endwind; 


           /* if wstarI is better sub here */
    if (wstarI_rss[ns]<wstarII_rss[ns]);
      wstarII_rss[ns] = wstarI_rss[ns];
      wstarII_beta[ns,.] = wstarI_beta[ns,.];
      wstarII_f[ns,.] = wstarI_f[ns,.];     /* WSTAR forecasts */
      wstarII_ci5[ns,.] = wstarI_ci5[ns,.]; wstarII_ci25[ns,.] = wstarI_ci25[ns,.]; wstarII_ci75[ns,.] = wstarI_ci75[ns,.]; wstarII_ci95[ns,.] = wstarI_ci95[ns,.];

    else;
      {sbc4,aic4} = sel_lag2(y00,xdet00,xtest[.,3:cols(xtest)],4);  /* used to get optimal lags */
      __selp1 = selif(aic4, aic4 .< 6);      /* asuming that selp = 1|2|3|5|6 */
      __selp2 = selif(aic4, aic4.> 6)-6; 
      selxtest = 1|2|(__selp1+2)|8|(__selp2+8);
      { d,d,beta_wstar_lin,d,d,d,d,d,d,res,d } = ols(0,y00,xtest_choice[.,selxtest]); 


      wstarII_beta[ns,.] = latest_wstarII_parest';
      {wstarII_f[ns,.],temp_ci} = WSTARfore_xx(y00,xdet00,xall00,xx00,xdet00_1saf,xall00_1saf,xx00_1saf,beta_wstar_lin,latest_wstarI_parest);     /* WSTAR forecasts */
      wstarII_ci5[ns,.] = temp_ci[1,.]; wstarII_ci25[ns,.] = temp_ci[2,.]; wstarII_ci75[ns,.] = temp_ci[3,.]; wstarII_ci95[ns,.] = temp_ci[4,.];

      __selp1 = selp;
      __selp2 = selp; 

    endif;

  ns = ns+1;    
          
endo;     /* end simulation loop */




/*=============
****
** procedures
****
=============*/



/*================================================================
  this procedure produces weighted LSTAR forecasts for
  10 periods ahead. Uses Monte Carlo to prodece
  expected values (400 simulations)

  input:    (i) y, dependent variable
            (ii) xdete, det vars estimation period
            (iii) xalle, lin explan var for est period, 1st col const
            (iv) tve, transition var, est period
            (v) xdetf, det var forecasting period
            (vi) xallf, lin expl var for 1 step ahead forecast
            (vii) tvf, transition var for 1 step ahead forecast
            (viii) beta, (10x1) vector of linear parameters
                    catering for AR(4), but some pars could be 0
            (ix) par, (4 x 1) parameter vector with 
                    gam and c parameter and 2 beta weight parameters
  output:   (i) fore, (1 x f) vector of expected values
                    for y, for periods T+1, T+2, ..., T+f
            (ii) the 5%, 25%, 75% and 95% percentiles for the forecasts
                    for y, for periods T+1, T+2, ..., T+f
                    (4 x f) matrix
==================================================================*/

proc (2) = WSTARfore_xx(y,xdete,xalle,tve,xdetf,xallf,tvf,beta,par);
local t,k,mcno,tv,gam0,c0,a,b,split,splitgrid,weights,f,
       xtest,sig,eps,fore,ey,e,qy;

t = rows(y);
mcno = 400;         /* no. of MC replications */ 

gam0 = par[1];
c0 = par[2];
a = par[3];
b = par[4];

            /* transform a and b into weights on [1,4] */
split = 1/(cols(tve)+1);
splitgrid = seqa(split,split,cols(tve));
a = maxc(a|0.001); a = minc(a|100);     /* ensure valid values for a and b */
b = maxc(b|0.001); b = minc(b|100);
weights = dbeta(splitgrid,a,b);
weights = weights/sumc(weights);

            /* calculate weighted transition variable */
tv = tve*weights;
f = 1/(1+exp(-(gam0*scale_gam)*(tv-c0)));  

xtest = xdete ~ xalle[.,__selp1] ~ f.*xdete[.,1] ~ f.*xalle[.,__selp2];


print "beta" beta';
print "cols in x =" (cols(xdete)~rows(__selp1)~1~rows(__selp2));
print "selp1 =" __selp1;
print "selp2 =" __selp2;


sig = stdc(y - xtest*beta);
eps = sig*rndn(1,mcno);

tv = tvf*weights;         /* calculate transition variable */
f = 1/(1+exp(-(gam0*scale_gam)*(tv-c0)));  

fore = (xdetf ~ xallf[.,__selp1] ~ f.*xdetf[.,1] ~ f.*xallf[.,__selp2])*beta + eps;

ey = meanc(fore');
e = 0.05|0.25|0.75|0.95;
qy = quantile(fore',e);
retp(ey',qy);
endp;



/*================================================================
  LSTARfore_xx(y,xdete,xalle,tve,xdetf,xallf,tvf,beta,par);
  this procedure produces standard LSTAR forecasts for
  10 periods ahead. Uses Monte Carlo to prodece
  expected values (400 simulations)
  allows for exogeneous explan. variables
  BUT ONLY ONE STEP AHEAD FORECASTING!!!!
  input:    (i) y, dependent variable
            (ii) xdete, det vars
            (iii) xalle, lin explan var for est period
            (iv) tve, transition var, est period
            (v) xdetf, det vars for 1 step ahead forecast
            (vi) xallf, lin expl var for 1 step ahead forecast
            (vii) tvf, transition var for 1 step ahead forecast
            (viii) beta, (10x1) vector of linear parameters
                    catering for AR(4), but some pars could be 0
            (ix) par, (2 x 1) parameter vector with 
                    gam and c parameter

  output:   (i) fore, (f x 1) vector of expected values
                    for y, for periods T+1, T+2, ..., T+f
            (ii) the 5%, 25%, 75% and 95% percentiles for the forecasts
                    for y, for periods T+1, T+2, ..., T+f
                    (4 x f) matrix
  global: scale_gam = scale parameter for trans variable
==================================================================*/

proc (2) = LSTARfore_xx(y,xdete,xalle,tve,xdetf,xallf,tvf,beta,par);
local t,mcno,gam0,c0,f,xtest,sig,eps,fore,ey,e,qy;

t = rows(y);
mcno = 400;         /* no. of MC replications */ 

gam0 = par[1];
c0 = par[2];

f = 1/(1+exp(-(gam0*scale_gam)*(tve-c0)));  
xtest = xdete ~ xalle[.,__selp1] ~ f.*xdete[.,1] ~ f.*xalle[.,__selp2];

sig = stdc(y - xtest*beta);
eps = sig*rndn(1,mcno);

f = 1/(1+exp(-(gam0*scale_gam)*(tvf-c0)));  
fore = (xdetf ~ xallf[.,__selp1] ~ f.*xdetf[.,1] ~ f.*xallf[.,__selp2])*beta + eps;

ey = meanc(fore');
e = 0.05|0.25|0.75|0.95;
qy = quantile(fore',e);
retp(ey',qy);
endp;


/*=========================================================
  this procedure produces linear AR forecasts for
  10 periods ahead
  input:    (i) y, dependent variable
            (ii) x, dependent var (matrix of lagged ys)
                    should include constant
            (iii)   beta, parameter vector, dimension to fit x
            (iv)    f, number of forecast periods
  output:   (i) fore, (f x 1) vector of expected values
                    for y, for periods T+1, T+2, ..., T+f

=========================================================*/

proc (1) = ARfore(y,x,beta,f);
local t,k,fore,i;

t = rows(y);
if cols(x) .ne rows(beta);
  print "Dimension of X matrix and parameter vector are not compliant!!!";
endif;
k = cols(x)-1; 
fore = y|zeros(f,1);

for i(t+1,t+f,1);
  fore[i] = (1~rev(fore[i-k:i-1])')*beta;
endfor;

retp(fore[t+1:rows(fore)]);
endp;


/*=========================================================
  ARfore_xx_mc(y,x,beta,f);
  this procedure produces linear AR forecasts for
  1 period ahead using MC simulation
  this allows for exog variables 
  input:    (i) y, dependent variable
            (ii) x, dependent var (matrix of lagged ys)
                    should include constant
            (iii) x for forecast period
            (iv)   beta, parameter vector, dimension to fit x
            
  output:   (i) fore, (f x 1) vector of expected values
                    for y, for periods T+1, T+2, ..., T+f
            (ii) the 5%, 25%, 75% and 95% percentiles for the forecasts
                    for y, for periods T+1, T+2, ..., T+f
                    (4 x f) matrix

=========================================================*/

proc (2) = ARfore_xx_mc(y,xe,xf,beta);
local mcno,t,k,sig,eps,fore,ey,e,qy;

mcno = 400;         /* no. of MC replications */ 
t = rows(y);
if cols(xe) .ne rows(beta);
  print "Dimension of X matrix and parameter vector are not compliant!!!";
endif;
k = cols(xe)-1; 
sig = stdc(y - xe*beta);
eps = sig*rndn(1,mcno);

fore = (xf*beta)+eps;


ey = meanc(fore');
e = 0.05|0.25|0.75|0.95;
qy = quantile(fore',e);
retp(ey',qy);
endp;


/*===============================================================
	this code calculates the weighted STAR
    linearity tests
    set the possible weight functions in
    betapar
    input: (i) y, dependent var
           (ii) x, explan var for linear part
           (iii) xdet, deterministic variables, incl const.
           (iii) xx, explan var for calculation of 
                    transition variable
    output: (i) (9x1) vector of p-values, max, ave, exp 
                for standard, hc robust and wb test version
            (ii) optbeta
	Ralf Becker, 16.2.2007
=================================================================*/

proc (2) = wSTAR(y,x,xdet,xx);
local n,bootno,kmax,v,betapar,weights,splitgrid,i,save_test_hc,save_test_hc_rep,save_test,save_test_rep,w,xw,xxw,xw2,xxw2,xw3,xxw3,x_alt,x_null,
b_null,res_null,b_alt,res_alt,rrr,mmm,mmm_inv,s_null,s_alt,vvv_null,vvv_alt,v_hat,middle_hc,middle_hc_2,
middle,middle_2,sig2_null,j,sv_hc,sv,temp,save_test_max,save_test_ave,save_test_exp,save_test_rep_max,
save_test_rep_ave,save_test_rep_exp,save_pval_max,save_pval_ave,save_pval_exp,save_test_hc_max,
save_test_hc_ave,save_test_hc_exp,save_pval_hc_max,save_pval_hc_ave,save_pval_hc_exp,optbeta,vwb,res_null_wb,
save_test_wb,save_test_wb_rep,save_pval_wb_max,save_pval_wb_ave,save_pval_wb_exp,sv_wb;

  n = rows(y);
  bootno = 1000;
  kmax = cols(xx);
  v = rndn(n,bootno);               /* random terms for draws from null distribution */
  vwb = ones(n,bootno);
  vwb = (v.>0).*vwb + (v.<=0).*(-1).*vwb;   /* random -1 or 1 values */

  if kmax == 4;
    betapar = 0.04~3 | 4~18 | 6~10 | 0.14~0.89 | 1~1 | 0.89~0.14 | 10~6 | 18~4 | 3~0.04 | 0.04~10 | 14~22 | 22~14 | 10~0.04;
  elseif kmax == 6;
    @betapar =  0.1~72 | 30~80 | 70~90 | 90~70 | 80~30 | 72~0.1;@   /* spikes only */
    betapar = 6~15 | 6~10 | 0.14~0.89 | 0.89~0.14 | 10~6 | 15~6 | 0.1~72 | 30~80 | 70~90 | 90~70 | 80~30 | 72~0.1; 
  elseif kmax == 8;
    betapar = 2~20 | 4~20 | 8~24 | 0.89~0.14 | 3~3 | 24~8 | 0.04~16 | 17~60 | 40~80 | 60~80 | 80~60 | 80~40 | 60~17;
    @betapar = 2~20 | 4~20 | 8~24 | 0.89~0.14 | 3~3 | 24~8;@
    @betapar = 0.04~16 | 17~60 | 40~80 | 60~80 | 80~60 | 80~40 | 60~17 |16~0.04;@
  elseif kmax == 12;
    betapar = 1~1|8~45|16~45|16~25|15~15|65~65|25~16|45~16|45~8;
    @betapar = 1~1|8~45|16~45|16~25|15~15|65~65|25~16|45~16|45~8|11~99|16~98|27~91|44~99|62~99|85~99|99~85|99~62|99~44|91~27|98~16|99~11;@
  else;
    print "ERROR: no appropriate weight vectors provided!!!";
  endif;

                        
  weights = zeros(kmax,rows(betapar));
  splitgrid = seqa(1/(kmax+1),1/(kmax+1),kmax);
                /* calculate weight matrix */
  for i(1,rows(betapar),1);
    temp = dbeta(splitgrid,betapar[i,1],betapar[i,2]);
    temp = temp./sumc(temp);
    weights[.,i] = temp; 
  endfor;

  save_test_hc = zeros(rows(betapar),1);
  save_test_hc_rep = zeros(rows(betapar),bootno);
  save_test_wb = zeros(rows(betapar),1);
  save_test_wb_rep = zeros(rows(betapar),bootno);
  save_test = zeros(rows(betapar),1);
  save_test_rep = zeros(rows(betapar),bootno);
    
  for w(1,rows(betapar),1);        /* loop over different weight vectors */
    xw = xx*weights[.,w];        /* generate weighted average */
    xxw = x.*xw;
    xw2 = xw.^2; xxw2 = x.*xw2;
    xw3 = xw.^3; xxw3 = x.*xw3;
                /* one big matrix of all variables in alternative */
    x_alt = xdet~x~xxw~xxw2~xxw3;
                /* one small matrix for the null model */
    x_null = xdet~x;

                /* define matrices, mainly as in Hansen */
    b_null = y/x_null; res_null = y - x_null*b_null;

    b_alt = y/x_alt; res_alt = y - x_alt*b_alt;
    rrr = (zeros(rows(b_alt)-rows(b_null),rows(b_null))~eye(rows(b_alt)-rows(b_null)))';
    mmm = x_alt'x_alt/n;
    mmm_inv = inv(mmm);
    s_null = x_alt .* res_null;
    s_alt = x_alt .* res_alt;
    vvv_null = s_null's_null/n; 
    vvv_alt = s_alt's_alt/n;
    v_hat = mmm_inv * vvv_null * mmm_inv;        /* LM test */
    middle_hc = rrr*inv(rrr'v_hat*rrr)*rrr';       
    middle_hc_2 = mmm_inv*middle_hc*mmm_inv;
    middle = rrr*inv(rrr'mmm_inv*rrr)*rrr';       
    middle_2 = mmm_inv*middle*mmm_inv;
    sig2_null = (res_null'res_null/n);


    save_test_hc[w] = n*b_alt'middle_hc*b_alt;     /* HC robust test */
    save_test[w] = n*b_alt'middle*b_alt/sig2_null;    /* non HC robust test */

  
            /* generate draws from the null distribution */
/*               @ loop version @
    for j(1,bootno,1);
@      sv_hc = s_alt.*v[.,j]/sqrt(n);            Wald test @
      sv_hc = s_null'v[.,j]/sqrt(n);            @ LM test @
      sv = x_alt'v[.,j]/sqrt(n);
      save_test_hc_rep[w,j] = sv_hc'middle_hc_2*sv_hc;         @ HC robust test @
      save_test_rep[w,j] = sv'middle_2*sv;         @ non HC robust test @
    endfor;
*/
                /* vectorised version, faster by a factor 4 to 5 */

                                 /* standard version */
    sv = (x_alt'v)/sqrt(n);      /* (k x bootno) matrix, equiv. of sv_hc */
    temp = sv'middle_2;          /* (bootno x k) matrix, equiv. of sv'middle_2 */
                                 /* the following line is equivalent to diag(sv'middle_2'sv)
                                    but much faster as the discarded off-diagonal calculations are avoided */
    save_test_rep[w,.] = sumc((temp.*sv')')';

                                 /* hc robust version */
    sv_hc = (s_null'v)/sqrt(n);  /* (k x bootno) matrix, equiv. of sv_hc */
    temp = sv_hc'middle_hc_2;    /* (bootno x k) matrix, equiv. of sv_hc'middle_hc_2 */
                                 /* the following line is equivalent to diag(sv_hc'middle_hc_2'sv_hc)
                                    but much faster as the discarded off-diagonal calculations are avoided */
    save_test_hc_rep[w,.] = sumc((temp.*sv_hc')')';

                                 /* wild bootstrap version */
    res_null_wb = res_null.*vwb;            /* res_null randomised by -1/1 two point distribution */
    sv_wb = (x_alt'res_null_wb)/sqrt(n);    /* (k x bootno) matrix, equiv. of sv_hc */
    temp = sv_wb'middle_hc_2;               /* (bootno x k) matrix, equiv. of sv_hc'middle_hc_2 */
                                            /* the following line is equivalent to diag(sv_hc'middle_hc_2'sv_hc)
                                                but much faster as the discarded off-diagonal calculations are avoided */
    save_test_wb_rep[w,.] = sumc((temp.*sv_wb')')';


  endfor;
                                                /* standard tests */
  save_test_max = maxc(save_test);            /* calculate data test stats */
  save_test_ave = meanc(save_test);
  save_test_exp = meanc(exp(0.5.*save_test));
  save_test_rep_max = maxc(save_test_rep);         /* calculate reps of test stats */
  save_test_rep_ave = meanc(save_test_rep);
  save_test_rep_exp = meanc(exp(0.5.*save_test_rep));
  save_pval_max = meanc(save_test_rep_max.>save_test_max);  /* estimate p values */
  save_pval_ave = meanc(save_test_rep_ave.>save_test_ave);
  save_pval_exp = meanc(save_test_rep_exp.>save_test_exp);

  optbeta = betapar[maxindc(save_test),.];

                                                /* HC robust tests */
  save_test_hc_max = maxc(save_test_hc);            /* calculate data test stats */
  save_test_hc_ave = meanc(save_test_hc);
  save_test_hc_exp = meanc(exp(0.5.*save_test_hc));
  save_test_rep_max = maxc(save_test_hc_rep);         /* calculate reps of test stats */
  save_test_rep_ave = meanc(save_test_hc_rep);
  save_test_rep_exp = meanc(exp(0.5.*save_test_hc_rep));
  save_pval_hc_max = meanc(save_test_rep_max.>save_test_hc_max);  /* estimate p values */
  save_pval_hc_ave = meanc(save_test_rep_ave.>save_test_hc_ave);
  save_pval_hc_exp = meanc(save_test_rep_exp.>save_test_hc_exp);

                                                /* Wild bootstrap tests */
                                                /* uses data test stats of HC case */
  save_test_rep_max = maxc(save_test_wb_rep);         /* calculate reps of test stats */
  save_test_rep_ave = meanc(save_test_wb_rep);
  save_test_rep_exp = meanc(exp(0.5.*save_test_wb_rep));
  save_pval_wb_max = meanc(save_test_rep_max.>save_test_hc_max);  /* estimate p values */
  save_pval_wb_ave = meanc(save_test_rep_ave.>save_test_hc_ave);
  save_pval_wb_exp = meanc(save_test_rep_exp.>save_test_hc_exp);


  temp = save_pval_max|save_pval_ave|save_pval_exp|save_pval_hc_max| 
  save_pval_hc_ave|save_pval_hc_exp|save_pval_wb_max|save_pval_wb_ave|save_pval_wb_exp;

retp(temp,optbeta);
endp;



/*==================================================
	calculates Terasvirta's test for nonlinearity
    with LSTAR as alternative
    Iterate over different delay parameters and
    use minimum p-value. Tera did not suggest this
    strategy as a testing strategy, but it is often done
	
	(Terasvirta, 1994)
	input:	(i) y, dependent var
			(ii) x, matrix of indep, without const.
            (iii) xdet, deterministic vars including const
            (iv) nlcols, index of cols to be considered for nonlinearity
	output: (i) p, p-value
            (ii) the best lag

	Ralf Becker, 4.12.2007
==================================================*/

proc (2) = tera(y,x,xdet,nlcols);
local t,p,x_null,xnl,pval_save,d,xd1,xd2,xd3,zd2,zd3,zd4,x_alt,res_null,res_alt,dof,teststat;

t = rows(x);
xnl = x[.,nlcols];

p = cols(xnl);

x_null = xdet~x;
res_null = y - x_null*(y/x_null);
pval_save = zeros(p,1);

for d(1,p,1);

  xd1 = xnl[.,d];
  xd2 = xnl[.,d].^2;
  xd3 = xnl[.,d].^3;

  zd2 = x *~ xd1;
  zd3 = x *~ xd2;
  zd4 = x *~ xd3;

  x_alt = x_null ~ zd2 ~ zd3 ~ zd4;

  res_alt = y - x_alt*(y/x_alt);

  dof = cols(x_alt)-cols(x_null); 
/*  
  teststat = ((res_null'res_null - res_alt'res_alt)/dof)/((res_null'res_null)/(t-p-dof-1));
  pval_save[d] = cdffc(teststat,dof,(t-p-dof-1));
*/
  teststat = t*((res_null'res_null - res_alt'res_alt))/((res_null'res_null));
  pval_save[d] = cdfchic(teststat,dof);


endfor;

retp(minc(pval_save),minindc(pval_save));
endp;


/*====================================================================
  criterion function to find WEIGHTED LSTAR 
  parameters when the transition function
  is allowed to be a weighted average
  of a variable. linear parameters are concentrated out
  
  input: (i) par (4x1) vector 
                const, linAR1, linAR2, fconst, fAR1, fAR2,
                 gam, c, a, b
                gam and c are parameters for the LSTAR fct
                a and b are the parameters for the beta fct
                determining the weights
         (ii) data, (univariate) data vector
  global:   (i)   __xx, matrix of explan variables for tv
            (ii)  __xdet, matrix of det vars, 1st col = constant
            (iii) __xall, matrix of explan variables for linear part (no det vars)
            (v)   maxlag, lags used for transition function 

  output: negative SSR (need a function to be maximised!)
   
======================================================================*/

proc (1) = wlstar_crit_conc_xx(par,data);
local gam,c,a,b,y,x,xdet,xx,tv,split,splitgrid,weights,f,xfull,res;
			/* unpacking parameter vector */

gam = par[1];
c = par[2];
a = par[3];
b = par[4];

y = data;
x = __xall;     /* this is for the linear part */
xdet = __xdet;  
xx = __xx;      /* xx is for the calculation of the transition variable */

            /* transform a and b into weights on [1,4] */
split = 1/(cols(xx)+1);
splitgrid = seqa(split,split,maxlag);
a = maxc(a|0.001); a = minc(a|100);     /* ensure valid values for a and b */
b = maxc(b|0.001); b = minc(b|100);
weights = dbeta(splitgrid,a,b);
weights = weights/sumc(weights);

            /* calculate weighted transition variable */
tv = xx*weights;
f = 1/(1+exp(-(gam*scale_gam)*(tv-c)));
xfull = xdet ~ x[.,__selp1] ~ xdet[.,1].*f ~ x[.,__selp2].*f;
            
res = y - xfull*invswp(xfull'xfull)*xfull'y;       /* residual */

retp(-(res'res));
endp;

/*=====================================================================================
  lstar_crit_conc_xx(par,data);
  criterion function to find STANDARD LSTAR 
  parameters. concentrated criterion function.
  explanatory variables can be exog.
  
  limitations so far:
    a) maximum AR lag is 4
    b) maxlag = 4
    c) no exogenous variables 
  input: (i) par (10x1) vector 
                gam, c, a, b
                gam and c are parameters for the LSTAR fct
                a and b are the parameters for the beta fct
                determining the weights
         (ii) data, (univariate) data vector (T+4) observations
                if arlag

  globals:  (i)   __slstarlag, delay parameter for transition variable
            (ii)  __xx, matrix of explan variables for tv
            (iii) __xall, matrix of explan variables for linear part (no det vars)
            (v)   __xdet, deterministic variables
  output: negative SSR (need a function to be maximised!)
   
=====================================================================================*/

proc (1) = lstar_crit_conc_xx(par,data);
local beta, gam, c, a,b,arlag,maxlag,y,sdy,x,xx,xdet,tv,split,splitgrid,weights,f,xfull,res,lag;

			/* unpacking parameter vector */
gam = par[1];
c = par[2];

lag = __slstarlag;          /* this is the delay parameter */

y = data;
x = __xall;
xx = __xx;
xdet = __xdet;

tv = xx[.,lag];            /* select transition variable taking the added constant into account */ 
          
f = 1/(1+exp(-(gam*scale_gam)*(tv-c)));
xfull = xdet ~ x[.,__selp1] ~ xdet[.,1].*f ~ x[.,selp2].*f;
            
res = y - xfull*invswp(xfull'xfull)*xfull'y;       /* residual */

retp(-(res'res));
endp;


/*=============================================================
  lstar_crit_conc(par,data);
  criterion function to find STANDARD LSTAR 
  parameters. concentrated criterion function.
  limitations so far:
    a) maximum AR lag is 4
    b) maxlag = 4
    c) no exogenous variables 
  input: (i) par (10x1) vector 
                gam, c, a, b
                gam and c are parameters for the LSTAR fct
                a and b are the parameters for the beta fct
                determining the weights
         (ii) data, (univariate) data vector (T+4) observations
                if arlag

  globals:  (i) __slstarlag, delay parameter for transition variable
            (ii) __arlag, order of AR process
  output: negative SSR (need a function to be maximised!)
   
==============================================================*/

proc (1) = lstar_crit_conc(par,data);
local beta, gam, c, a,b,arlag,maxlag,y,sdy,x,tv,split,splitgrid,weights,f,xfull,res,lag;

			/* unpacking parameter vector */
gam = par[1];
c = par[2];

lag = __slstarlag;          /* this is the delay parameter */
arlag = __arlag;            /* this is the AR order */

{y,x} = embed(data,4,1,1);  /* first generate 4 lags */
x = ones(rows(y),1)~x;
tv = x[.,lag+1];            /* select transition variable taking the added constant into account */ 
x = x[.,1:arlag+1];         /* adjust to required AR length */      
            
f = 1/(1+exp(-(gam*scale_gam)*(tv-c)));
xfull = x ~ x.*f;
            
res = y - xfull*invswp(xfull'xfull)*xfull'y;       /* residual */

retp(-(res'res));
endp;


/*=============================================================
  criterion function to find STANDARD LSTAR 
  parameters when the transition function
  is allowed to be a weighted average
  of a variable 
  limitations so far:
    a) only linear AR(2)
    b) maxlag = 8
    c) no exogenous variables 
  input: (i) par (10x1) vector 
                const, linAR1, linAR2, fconst, fAR1, fAR2,
                 gam, c, a, b
                gam and c are parameters for the LSTAR fct
                a and b are the parameters for the beta fct
                determining the weights
         (ii) data, (univariate) data vector
  output: negative SSR (need a function to be maximised!)
   
==============================================================*/

proc (1) = lstar_crit(par,data);
local beta, gam, c, a,b,maxlag,y,sdy,x,tv,split,splitgrid,weights,f,xfull,res,lag;
			/* unpacking parameter vector */
beta = par[1:6];
gam = par[7];
c = par[8];
maxlag = 8;
lag = __slstarlag;      /* this is the delay parameter */

y = trimr(data,maxlag,0);

x = ones(rows(y),1)~trimr(data,7,1)~trimr(data,6,2);
tv = trimr(data,(maxlag-lag),lag); 
            
f = 1/(1+exp(-(gam*scale_gam)*(tv-c)));
xfull = x ~ x.*f;
            
res = y - xfull*beta;       /* residual */

retp(-(res'res));
endp;




/*==================================================
	hcse(y,x)	
	this procedure calculates heteroscedasticity
	consistent standard errors for regression
	coefficients
	input:	(i)		y, dep data
			(ii)	x, indep data
	output:	(i) 	kx4 matrix of
					- parameter estimates
					- Newey West se
					- White se
					- OLS se
	The lag length for the Newey-West window is
	set to int(4(n/100)^(2/9))
==================================================*/
proc (1) = hcse(y,x);

local n,k,b,e,ex,g,i,l,la,w,za,f,inf,v,nwerr,wherr,olserr;

n = rows(x);
k = cols(x);
b = y/x;
e = y - x*b;
ex = e.*x;
la = int(4*(n/100)^(2/9));
				/* Newey-West */
for i(0,la,1);
  l = i;
  w = (la+1-i)/(la+1);
  za = trimr(ex,l,0)'trimr(ex,0,l);
  if i==0; g = za;
  else; g = g+w*(za+za');
  endif;
endfor;

f = x'*x;
inf = inv(f);
v = inf*g*inf;
nwerr = (diag(v))^.5;

				/* White */

wherr = inf*(ex'ex)*inf;
wherr = (diag(wherr))^.5;

olserr=(diag(inv(x'x)*stdc(e)^2))^.5;
retp(b~nwerr~wherr~olserr);

endp;



/*=======================================
  helper fct to find beta 
  starting values
  input; (i) par, 2x1 vector with a and b
         (ii) discrete weight vector to 
                which to fit beta 
========================================*/

proc optbetaweight(par,weights);
local a,b,split, splitgrid,test,test_rw,maxfun;

a = par[1];
b = par[2];
split = 1/(rows(weights)+1);
splitgrid = seqa(split,split,rows(weights));

test = dbeta(splitgrid,a,b);
test_rw = test./sumc(test);

maxfun = -(weights-test_rw)'(weights-test_rw);

retp(maxfun);
endp;


/*=================================
**    Density of the BETA distribution
**  y=dbeta(x,a,b)   
**  Input
**  -----------------------------------------------------------------
**  x...(k,m) matrix, 0<x<1
**  a...scalar, >0
**  b...scalar, >0
**  y...f(x), (k,m)-matrix, values of the density
=================================*/
proc dbeta(x,a,b);
 local xx,d;
  a = abs(a); b = abs(b);
  xx = x.*(0.<=x).*(x.<=1);
   d = xx^(a-1).*(1-xx)^(b-1)*GAMMA(a+b)/(GAMMA(a)*GAMMA(b));
 retp(d.*(0.<=x).*(x.<=1));
endp;


/*===================================================
	This procedure selects the optimal lag structure
	for an AR model
	inputs:	(i)		dat, data series
			(ii)	k, maximal lag
	output:	(i)		sic, lags selected from SIC
			(ii)	aic, lags selected from AIC
	this procedure requires the embed procedure

	Ralf Becker, 1 Sept 2005
====================================================*/	

proc(2) = sel_lag1(dat,k);

local q,m,i,l,a,sic,aic,test,lags,y,x,
		n,xc,e,sde,sels,sela;


sic = zeros(k+1,1);
aic = sic;
test = sic;

for i(1,k+1,1);
  lags = i-1;
  if lags > 0;
    {y,x} = embed(dat,lags,1,1);
    n = rows(y);
    xc = ones(n,1)~x;
  else;				/* this is the const only case */
    y = dat;
	n = rows(dat);
	xc = ones(n,1);
  endif;	
  e = y - xc*(y/xc);
  sde = sqrt(e'e/n); 
  test[i] = lags;
  sic[i] = (ln(sde^2) + (lags+1) * ln(n) / n);
  aic[i] = ln(sde^2) + 2 * (lags+1) / n;
  
endfor;

sels = sortc(sic~test,1);
sic = sels[1,2];
sela = sortc(aic~test,1);
aic = sela[1,2];

retp(sic,aic);
endp;

/*===================================================
	This procedure selects the optimal lag structure
	for a regression model, this selects individual 
    elements from datx
	inputs:	(i)		y, data series
            (ii)    xdet, deterministic variables or 
                        variables to be certainly included
            (iii)   datx, matrix of x
            (iv)    addpar, how many extra parameters
	output:	(i)		sic, lags selected from SIC
			(ii)	aic, lags selected from AIC
	this procedure requires the embed procedure

	Ralf Becker, 18 Nov 2007
====================================================*/	

proc(2) = sel_lag2(y,xdet,datx,addpar);

local q,m,i,l,a,sic,aic,test,lags,x,
		n,xc,e,sde,sels,sela,k;

k = cols(datx);
n = rows(y);
q = 2^(k); 		/* number of combinations */
m = zeros(q,k);

				/* creates matrix of possible combinations */
for i(1,k,1);
  l = q/(2^i);
  a = (ones(l,1)*i)|zeros(l,1);
  m[.,i] = reshape(a,q,1);
endfor;

sic = zeros(q,1);
aic = sic;
test = sic;

for i(1,q,1);
  lags = m[i,.]';
  if (sumc(lags) ne 0);
    lags = selif(lags, lags .ne 0);
    x = datx[.,lags];
    xc = xdet~x;
  else;				/* this is the const only case */
	xc = xdet;
  endif;	

  e = y - xc*(y/xc);
  sde = sqrt(e'e/n); 
  lags = cols(xc); test[i] = lags;
  sic[i] = (ln(sde^2) + (lags+1+addpar) * ln(n) / n);
  aic[i] = ln(sde^2) + 2 * (lags+1+addpar) / n;
  
endfor;

sels = sortc(sic~seqa(1,1,q)~test,1);
sic = m[sels[1,2],.];
sic = selif(sic',(sic' .ne 0));
sela = sortc(aic~seqa(1,1,q)~test,1);
aic = m[sela[1,2],.];
aic = selif(aic',(aic' .ne 0));

retp(sic,aic);
endp;


/*=======================================================
	proc to embed data
	input: 	(i)		z, data vector
			(ii)	k, embed dimension
					if scalar it creates all lags up to k
					if nx1 vector it creates the n lags
			(iii)	tau, embed lag
			(iv)	seq, if 1 then lags = 1 .. k
						 if 0 then lags = {k}
	output:	(i)		y, image
			(ii)	x, embeded lags
    x only contains a constant if k == 0;
	Ralf Becker, 18.5.2000
=======================================================*/

proc (2) = embed(z,k,tau,seq);

local yx,j,x,y,max;


if (seq==1);
  yx = trimr(z,k*tau,0);
  for j(tau,k*tau,tau);
    yx = yx ~ trimr(z,(k*tau-j),j); 
  endfor;
  y = yx[.,1];
  if (k ne 0); x = yx[.,2:cols(yx)];
  else; x = ones(rows(y),1);
  endif;
else;		/* if selected lags */
  max = maxc(k);
  yx = trimr(z,max*tau,0);
  for j(1,rows(k),1);
    yx = yx ~ trimr(z,(max*tau-k[j]*tau),k[j]*tau); 
  endfor;
  y = yx[.,1];
endif;

if (k ne 0); x = yx[.,2:cols(yx)];
else; x = ones(rows(y),1);
endif;

retp(y,x);
endp;

/*============================================
  procedure to plot beta distribution
  input:    (i) ab, 2x1 beta parameter vector
            (ii) kmax. maximum lag used
            (iii) graph title
==============================================*/

proc (0) = betaplot(ab,kmax,tit);
 local a,b,splitgrid,temp,betapar2;
 a = ab[1];
 b = ab[2];

 kmax = kmax + 1;       
 splitgrid = seqa(1/kmax,1/kmax,kmax-1);

 betapar2 = 99~62;

 

 temp = dbeta(splitgrid,a,b);
 temp = temp./sumc(temp);

 _pltype = {6,6,6,6,6,6,6,6};
 ytics(0.0,1.0,0.25,0); 
 xtics(1.0,kmax-1,1.0,0);

 _pmcolor = {15};
 _pcolor = {15};
 title(tit);
 xy(seqa(1,1,kmax-1),temp);

endp;

